#include "nrutil.h"

						 /*  MODULE 3.5       */
void polcoe(float x[], float y[], int n, float cof[])
                                                                      /*
3.5 COEFFICIENTS OF THE INTERPOLATING POLYNOMINAL

VERSION  1 04/Sept/92 Krakow  updated by Tomasz Twardowski

Given arrays x[1..n] and y[1..n] containing a tabulated function
y[i]=f(x[i]), this routine returns an array of coefficient cof
[1..n], such thet y[i]=ADDj (cof[j]*(x[i]^(j-1)) )                    */
{
int      k, j, i;
float    phi, ff, b, *s;
                                                                      /*
------------------------------------------ POLCOE - BEGIN ----------- */

    s = vector(0, n);
    for (i = 0; i <= n; i++)
       s[i] = cof[i] = 0.0;
    s[n] = -x[0];
                                                                      /*
coefficients a, of the master polynomial p(x) are found by recurence  */
    for (i = 1; i <= n; i++)
    {
       for (j = n - i; j <= n - 1; j++)
          s[j] -= x[i] * s[j + 1];
       s[n] -= x[i];
    }
    for (j = 0; j <= n; j++)
    {
       phi = n + 1;
                                                             /*  the 
quantity phi=mulj<>k (x[j]-x[k]) is found as a derivative of p(x[j])  */
       for (k = n; k >= 1; k--)
          phi = k * s[k] + x[j] * phi;
       ff = y[j] / phi;
       b = 1.0;
                                     /*  coefficients of polynomials 
in each term of the lagrange formula are found by synthetic 
division of p(x) by (x-x[j]). the solution cof[k] is accumulated      */
       for (k = n; k >= 0; k--)
       {
          cof[k] += b * ff;
          b = s[k] + x[j] * b;
       }
    }
    free_vector(s, 0, n);
                                                                      /*
------------------------------------------ POLCOE - END ------------- */
} 