#define kkk 2.0
#define SWAP(g,h) {y=(g);(g)=(h);(h)=y;}
#define SIGN(a,b) ((b) > 0 ? fabs(a) : -fabs(a))

#include <stdlib.h>
#include <math.h>
#include <stdio.h>

                                                 /*  MODULE 11.6      */
void hqr(double **a, int n, double wr[], double wi[])
                                                                      /*
11.6 THE QR ALGORITHM FOR REAL HESSENBERG MATRICES.

VERSION  1 04/Sept/92 Krakow updated by Tomasz Twardowski

Finds all eigenvalues of an upper Hessenberg matrix a[1..n][1..n]. 
On input a can be exactly as output from elmhes; on output it is 
destroyed. The real and imaginary parts of the eigenvalues are 
returned in wr[1..n] and wi[1..n] respectively.                       */
{
int      nn, m, l, k, j, its, i, mmin;
double    z, y, x, w, v, u, t, s, r, q, p, anorm;
//void     nrerror();
                                                                      /*
--------------------------------------------- HQR - BEGIN ----------- */

                                                                      /*
1. COMPUTE MATRIX NORM FOR POSSIBLE USE IN LOCATING SINGLE SMALL 
   SUBDIAGONAL ELEMENT.
                                                                      */
    anorm = fabs(a[1][1]);
    for (i = 2; i <= n; i++)
       for (j = (i - 1); j <= n; j++)
          anorm += fabs(a[i][j]);
    nn = n;
    t = 0.0;
                    /*  ...gets changed only by an exceptional shift  */
                                                                      /*
2. LOOP OF SEARCHING FOR NEXT EIGENVALUE.
                                                                      */
    while (nn >= 1)
    {
       its = 0;
       do
       {
                                                                      /*
2.1. ITERATION ON CURRENT EIGENVALUE.
                                                                      */
                      /*  look for single small subdiagonal element.  */
          for (l = nn; l >= 2; l--)
          {
             s = fabs(a[l - 1][l - 1]) + fabs(a[l][l]);
             if (s == 0.0)
                s = anorm;
             if (fabs(a[l][l - 1]) + s == s)
                break;
          }
          x = a[nn][nn];
          if (l == nn)
          {
                                                  /*  one root found  */
             wr[nn] = x + t;
             wi[nn--] = 0.0;
          }
          else
          {
             y = a[nn - 1][nn - 1];
             w = a[nn][nn - 1] * a[nn - 1][nn];
             if (l == (nn - 1))
             {
                                                 /*  two roots found  */
                p = 0.5 * (y - x);
                q = p * p + w;
                z = sqrt(fabs(q));
                x += t;
                if (q >= 0.0)
                {
                                                     /*  a real pair  */
                   z = p + SIGN(z, p);
                   wr[nn - 1] = wr[nn] = x + z;
                   if (z)
                      wr[nn] = x - w / z;
                   wi[nn - 1] = wi[nn] = 0.0;
                }
                else
                {
                                                  /*  a complex pair  */
                   wr[nn - 1] = wr[nn] = x + p;
                   wi[nn - 1] = -(wi[nn] = z);
                }
                nn -= 2;
             }
             else
             {
                             /*  no roots found. continue iteration.  */
                if (its == 30)
                   //nrerror("Too many iterations in HQR");
		   fprintf(stderr, "Too many iterations in HQR\n");
                if (its == 10 || its == 20)
                {
                                          /*  form exceptional shift  */
                   t += x;
                   for (i = 1; i <= nn; i++)
                      a[i][i] -= x;
                   s = fabs(a[nn][nn - 1]) + fabs(a[nn - 1][nn - 2]);
                   y = x = 0.75 * s;
                   w = -0.4375 * s * s;
                }
                ++its;
                for (m = (nn - 2); m >= l; m--)
                {
                                                                      /*
form shift. then look for 2 consecutive small subdiagonal elements.   */
                   z = a[m][m];
                   r = x - z;
                   s = y - z;
                                                /*  equation 11.6.23  */
                   p = (r * s - w) / a[m + 1][m] + a[m][m + 1];
                   q = a[m + 1][m + 1] - z - r - s;
                   r = a[m + 2][m + 1];
                          /*  scale to prevent overflow or underflow  */
                   s = fabs(p) + fabs(q) + fabs(r);
                   p /= s;
                   q /= s;
                   r /= s;
                   if (m == l)
                      break;
                   u = fabs(a[m][m - 1]) * (fabs(q) + fabs(r));
                                                /*  equation 11.6.26  */
                   v = fabs(p) * (fabs(a[m - 1][m - 1]) + fabs(z) +
                       fabs(a[m + 1][m + 1]));
                   if (u + v == v)
                      break;
                }
                for (i = m + 2; i <= nn; i++)
                {
                   a[i][i - 2] = 0.0;
                   if (i != (m + 2))
                      a[i][i - 3] = 0.0;
                }
                                                                      /*
2.1.1. DOUBLE QR STEP ON ROWS 1 TO NN AND COLUMNS M TO NN.
                                                                      */
                for (k = m; k <= nn - 1; k++)
                {
                   if (k != m)
                   {
                              /*  begin setup of householder vector.  */
                      p = a[k][k - 1];
                      q = a[k + 1][k - 1];
                      r = 0.0;
                      if (k != (nn - 1))
			 r = a[k + 2][k - 1];
		      x = fabs(p) + fabs(q) + fabs(r);
		      if (x)
                      {
                         /*  scale to prevent overflow or underflow.  */
                         p /= x;
                         q /= x;
                         r /= x;
                      }
		   }
		   s = SIGN(sqrt(p * p + q * q + r * r), p);
		   if (s)
                   {
                      if (k == m)
                      {
                         if (l != m)
                            a[k][k - 1] = -a[k][k - 1];
                      }
                      else
                         a[k][k - 1] = -s * x;
                                                /*  equation 11.6.24  */
                      p += s;
                      x = p / s;
                      y = q / s;
                      z = r / s;
                      q /= p;
                      r /= p;
                                                                      /*
2.1.2. ROW MODIFICATION.
                                                                      */
                      for (j = k; j <= nn; j++)
                      {
                         p = a[k][j] + q * a[k + 1][j];
                         if (k != (nn - 1))
                         {
                            p += r * a[k + 2][j];
                            a[k + 2][j] -= p * z;
                         }
                         a[k + 1][j] -= p * y;
                         a[k][j] -= p * x;
                      }
                      mmin = nn < k + 3 ? nn : k + 3;
                                                                      /*
2.1.3. COLUMN MODIFICATION.
                                                                      */
                      for (i = l; i <= mmin; i++)
                      {
                         p = x * a[i][k] + y * a[i][k + 1];
                         if (k != (nn - 1))
                         {
                            p += z * a[i][k + 2];
                            a[i][k + 2] -= p * r;
                         }
                         a[i][k + 1] -= p * q;
                         a[i][k] -= p;
                      }
                   }
                }
                     /*  go for next iteration on current eigenvalue  */
             }
          }
       }
       while (l < nn - 1);
    }
                                     /*  go back for next eigenvalue  */
                                                                      /*
--------------------------------------------- HQR - END ------------- */
} 
                              
						 /*  MODULE 11.5      */
void elmhes(double **a, int n)
                                                                      /*
11.5 REDUCTION OF A GENERAL MATRIX TO HESSENBERG FORM.

VERSION  1 04/Sept/92 Krakow updated by Tomasz Twardowski

Reduction to Hessenberg form by the elimination method. The real, 
nonsymmetric matrix a[1..n][1..n] is replaced by an upper Hessenberg 
matrix with identical eigenvalues. Recommended, but not required, is 
that this routine be preceded by balanc. On output, the Hessenberg 
matrix is in elements a[i][j] with i<=j+1. Elements with i>j+1 are 
to be thought of as zero, but are returned with random values.        */
{
int      m, j, i;
double    y, x;
                                                                      /*
------------------------------------------ ELMHES - BEGIN ----------- */

    for (m = 2; m < n; m++)
    {
       x = 0.0;
       i = m;
                                                                      /*
1. FIND THE PIVOT.
                                                                      */
       for (j = m; j <= n; j++)
       {
          if (fabs(a[j][m - 1]) > fabs(x))
          {
             x = a[j][m - 1];
             i = j;
          }
       }
                                                                      /*
2. INTERCHANGE ROWS AND COLUMNS.
                                                                      */
       if (i != m)
       {
          for (j = m - 1; j <= n; j++)
             SWAP(a[i][j], a[m][j])
          for (j = 1; j <= n; j++)
             SWAP(a[j][i], a[j][m])
       }
                                                                      /*
3. CARRY OUT THE ELIMINATION.
                                                                      */
       if (x)
       {
          for (i = m + 1; i <= n; i++)
	  {
	     y = a[i][m -1];
	     if (y)
             {
                y /= x;
                a[i][m - 1] = y;
                for (j = m; j <= n; j++)
                   a[i][j] -= y * a[m][j];
                for (j = 1; j <= n; j++)
                   a[j][m] += y * a[j][i];
             }
          }
       }
    }
                                                                      /*
------------------------------------------ ELMHES - END ------------- */
} 

double norma(double* wektor,int n) {
	int i;
	double wynik = 0;

    	for (i = 0; i < n; i++)
		wynik += wektor[i] * wektor[i];

	return sqrt(wynik);
}

int warunek1(double* xPoprzednie,double* xNowe,int n,double war) {
	double* wynik = (double*)malloc(n*sizeof(double));
	int i;

    	for (i = 0; i < n; i++)
		wynik[i] = xNowe[i] - xPoprzednie[i];

	if (norma(wynik,n) < war)
		i = 1;
	else
		i = 0;
		
	free(wynik);
	return i;
}

int warunek2(double** A,double* xNowe,int n,double PromienSpektralny, double* b) {
	double* wynik = (double*)calloc(n, sizeof(double));
	int i,j;

    	for (i = 0; i < n; i++)
		for (j = 0; j < n; j++)
			wynik[i] += A[i][j] * xNowe[j];

	for (i = 0; i < n; i++)
		wynik[i] = wynik[i] - b[i];

	if ((norma(wynik,n)/norma(b,n)) < PromienSpektralny)
		i = 1;
	else
		i = 0;
		
	free(wynik);
	return i;
}

void wyliczMacierz(int n, double** A) {
	int i;

	for (i = 0 ; i < n ;i++)
		bzero(A[i], n*sizeof(double));

	for (i = 1; i < n-1 ;i++)
		A[i][i] = kkk;

	for (i = 0; i < n-1;i++) {
		A[i][i+1] = 1.0/(i+2.0);
		A[i+1][i] = 1.0/(i+2.0);
	}

	A[0][0] = 1;
	A[n-1][n-1] = 1;
}

double promienSpektralny(double**A, int n) {
	double** tmpA = (double**)malloc((n+1)*sizeof(double*));
	int i, j;
	double max = 0;
	double*wwlasne_r = (double*)malloc((n+1)*sizeof(double));
	double*wwlasne_i = (double*)malloc((n+1)*sizeof(double));
	
	for (i = 1 ; i <= n; i++)
		tmpA[i] = (double*)malloc((n+1)*sizeof(double));
			
	for (i = 0 ; i < n; i++) {
		for (j = 0 ; j < n; j++)
			tmpA[j+1][i+1] = -1.0/A[i][i]*A[j][i];
		tmpA[i+1][i+1] += 1;
	}

	elmhes(tmpA, n);
	hqr(tmpA, n, wwlasne_r, wwlasne_i);
	
	for (i = 1 ; i <= n; i++)
		if (sqrt(wwlasne_r[i]*wwlasne_r[i]+wwlasne_i[i]*wwlasne_i[i])>max)
			max = sqrt(wwlasne_r[i]*wwlasne_r[i]+wwlasne_i[i]*wwlasne_i[i]);
		
	free(wwlasne_r);
	free(wwlasne_i);
	for (i = 1 ; i <= n; i++)
		free(tmpA[i]);
	free(tmpA);

	return max;
}

int main() {
	int i,j, n;
	int LiczbaIteracji = 0;
	double PromienSpektralny;
	double WarunekKonca;
	double** x = (double**)malloc(2*sizeof(double*));
	double** A;
	double* b;
	double omega = 1;
	double suma, e = 0;
	double*xtmp;

	printf("Podaj wymiar macierzy: ");
	scanf("%d", &n);

	xtmp = malloc(n*sizeof(double));
	b = calloc(n, sizeof(double));

	printf("Podaj wartosci elementow wektora x:\n");
	for (i = 0; i < n; i++)
		scanf("%lf", &xtmp[i]);
		
	printf("Podaj dokladnosc: ");
	scanf("%lf", &WarunekKonca);

	x[0] = (double*)calloc(n,sizeof(double));
	x[1] = (double*)calloc(n,sizeof(double));

	A=(double**)malloc(n*sizeof(double*));
	for (i = 0 ; i < n; i++)
		A[i] = (double*)malloc(n*sizeof(double));
		
	wyliczMacierz(n,A);
	
	for (i=0; i<n; i++)
		for (j=0; j<n; j++)
			b[i] += xtmp[j]*A[i][j];
	
	PromienSpektralny = promienSpektralny(A, n);
	printf("\nPromien spektralny macierzy iteracji Jacobiego: %f\n\n", PromienSpektralny);
	printf("Dla metody Czebyszewa:\n");
	printf("Dla pierwszrgo warunku stopu:\n");
	
	do {
    		LiczbaIteracji++;

    		for ( i = 0; i < n; i++ )
			x[0][i] = x[1][i];
		
    		for (i = 0; i < n;i++) {
			suma = b[i];
	
			for (j = 0; j < i;j++)
				suma -= A[i][j] * x[1][j];
	
			for (j = i+1; j < n;j++)
				suma -= A[i][j] * x[0][j];
	
			suma *= omega / A[i][i];
			x[1][i] += suma;
	
			x[1][i] -= x[0][i] * omega;
		}
    
    		if (LiczbaIteracji == 1)
			omega = 1.0 / (1 - 0.5*omega*PromienSpektralny*PromienSpektralny);
    		else
			omega = 1.0 / (1 - 0.25*omega*PromienSpektralny*PromienSpektralny);
  	} while (!warunek1(x[0],x[1],n,WarunekKonca));
	
	printf(" liczba iteracji: %d\n", LiczbaIteracji);
	printf(" wektor x:\n");
	for(i=0; i<n; i++)
		printf("  %20.20f\n", x[1][i]);
	for (j=0; j<n; j++)
		e += fabs(xtmp[j] - x[1][j]);
	e /= n;
	printf(" sredni blad bezwzgledny: %20.20f\n", e);
	
	bzero(x[1], n*sizeof(double));
	LiczbaIteracji = 0;
	omega = 1;
	do {
    		LiczbaIteracji++;

    		for ( i = 0; i < n; i++ )
			x[0][i] = x[1][i];
    
    		for (i = 0; i < n;i++) {
			suma = b[i];
	
			for (j = 0; j < i;j++)
				suma -= A[i][j] * x[1][j];
	
			for (j = i+1; j < n;j++)
				suma -= A[i][j] * x[0][j];
	
			suma *= omega / A[i][i];
				x[1][i] += suma;
	
			x[1][i] -= x[0][i] * omega;
		}
    
    		if (LiczbaIteracji == 1)
			omega = 1.0 / (1 - 0.5*omega*PromienSpektralny*PromienSpektralny);
    		else
			omega = 1.0 / (1 - 0.25*omega*PromienSpektralny*PromienSpektralny);
  	} while (!warunek2(A,x[1],n,WarunekKonca, b));

	printf("Dla drugiego warunku stopu:\n");
	printf(" liczba iteracji: %d\n", LiczbaIteracji);
	printf(" wektor x:\n");
	for(i=0; i<n; i++)
		printf("  %20.20f\n", x[1][i]);
	e = 0;
	for (j=0; j<n; j++)
		e += fabs(xtmp[j] - x[1][j]);
	e /= n;
	printf(" sredni blad bezwzgledny: %20.20f\n", e);
	
	printf("\nDla metody Jacobiego:\n");
	
	bzero(x[1], n*sizeof(double));
	LiczbaIteracji = 0;
	do {
    		LiczbaIteracji++;

    		for ( i = 0; i < n ; i++ )
			x[0][i] = x[1][i];
	
    		for ( i = 0; i < n ; i++ ) {
			suma = 0;
			for ( j = 0; j < n; j++)
				if (i!=j) suma += A[i][j] * x[0][j];

			suma = b[i] - suma;
		
			if (!A[i][i])
				fprintf(stderr,"Blad\n");
			x[1][i] = suma / A[i][i];
        
		}	
	} while (!warunek1( x[0], x[1], n, WarunekKonca));

	printf("Dla pierwszego warunku stopu:\n");
	printf(" liczba iteracji: %d\n", LiczbaIteracji);
	printf(" wektor x:\n");
	for(i=0; i<n; i++)
		printf("  %20.20f\n", x[1][i]);
	e = 0;
	for (j=0; j<n; j++)
		e += fabs(xtmp[j] - x[1][j]);
	e /= n;
	printf(" sredni blad bezwzgledny: %20.20f\n", e);
		
	bzero(x[1], n*sizeof(double));
	LiczbaIteracji = 0;
    	do {
    		LiczbaIteracji++;

    		for ( i = 0; i < n ; i++ )
			x[0][i] = x[1][i];
	
    		for ( i = 0; i < n ; i++ ) {
			suma = 0;
			for ( j = 0; j < n; j++)
				if (i!=j) suma += A[i][j] * x[0][j];

			suma = b[i] - suma;
		
			if (!A[i][i])
				fprintf(stderr,"Blad\n");
			x[1][i] = suma / A[i][i];
        
		}	
	} while (!warunek2( A, x[1], n, WarunekKonca, b));
    
	printf("Dla drugiego warunku stopu:\n");
	printf(" liczba iteracji: %d\n", LiczbaIteracji);
	printf(" wektor x:\n");
	for(i=0; i<n; i++)
		printf("  %20.20f\n", x[1][i]);
	e = 0;
	for (j=0; j<n; j++)
		e += fabs(xtmp[j] - x[1][j]);
	e /= n;
	printf(" sredni blad bezwzgledny: %20.20f\n", e);
		
	return EXIT_SUCCESS;
}
